﻿/*
 * The MIT License (MIT)
Copyright (c) 2013 João Tiago Pinheiro Neto Jacob
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*
*/
using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Player : MonoBehaviour
{
    [HideInInspector]
    public float x;
    [HideInInspector]
    public float z;
    [HideInInspector]
    public float y;
    [HideInInspector]
    public int zone;
    public bool useGPS;
    public double initialFakeLat;
    public double initialFakeLon;
    public double fakeLat;
    public double fakeLon;
    public float fakeSpeed = 0;
    public float fakeSpeedIncrement;
    public float fakeSpeedDecrement;
    public float maxFakeSpeed;
    public float fakeRotation;
    [HideInInspector]
    public GeoUTMConverter playerConverter;
    [HideInInspector]
    public bool isReady = true;


    // Use this for initialization
    public void Start()
    {
        Input.compass.enabled = true;
        playerConverter = new GeoUTMConverter();
        transform.position = new Vector3(0, 1.5f, 0);
        if (useGPS)
        {
            Input.location.Start();
        }
        else
        {
            
            playerConverter.ToUTM(initialFakeLat, initialFakeLon);
            x = (float)(playerConverter.X / GeoUTMConverter.Precision);
            z = (float)(playerConverter.Y / GeoUTMConverter.Precision);
            zone = (int)playerConverter.Zone;

        }
    }

    // Update is called once per frame
    public void Update()
    {
        if (useGPS)
        {
            LocationInfo li = Input.location.lastData;
            playerConverter.ToUTM((double)li.latitude, (double)li.longitude);
            x = (float)(playerConverter.X / GeoUTMConverter.Precision);
            z = (float)(playerConverter.Y / GeoUTMConverter.Precision);
            zone = (int)playerConverter.Zone;
            transform.position = new Vector3(0, 1.5f, 0);
            fakeLat = li.latitude;
            fakeLon = li.longitude;
        }
        else
        {
            if (Application.platform == RuntimePlatform.Android)
            {
                Input.compensateSensors = true;
                if (Input.GetTouch(0).phase == TouchPhase.Stationary || Input.GetTouch(0).phase == TouchPhase.Moved)
                {
                    if (fakeSpeed < -this.maxFakeSpeed)
                    {
                        fakeSpeed =-maxFakeSpeed;
                    }
                    else
                    {
                        fakeSpeed += -fakeSpeedIncrement;
                    }

                    rigidbody.velocity = fakeSpeed * transform.forward;
                }

                transform.localEulerAngles = new Vector3(Input.acceleration.z + transform.localEulerAngles.x, Input.acceleration.x + transform.localEulerAngles.y, Input.acceleration.x * 30);

            }

            if (Input.GetKey(KeyCode.A))
            {
                transform.eulerAngles = new Vector3(transform.rotation.eulerAngles.x, transform.rotation.eulerAngles.y - fakeRotation, transform.rotation.eulerAngles.z);

            }
            if (Input.GetKey(KeyCode.D))
            {

                transform.eulerAngles = new Vector3(transform.rotation.eulerAngles.x, transform.rotation.eulerAngles.y + fakeRotation, transform.rotation.eulerAngles.z);
            }
            if (Input.GetKey(KeyCode.W))
            {
                if (fakeSpeed > this.maxFakeSpeed)
                {
                    fakeSpeed = maxFakeSpeed;
                }
                else
                {
                    fakeSpeed += fakeSpeedIncrement;
                }

                rigidbody.velocity = fakeSpeed * transform.forward;
                //transform.position = transform.position - fakeSpeed*transform.forward;

            }

            if (Input.GetKey(KeyCode.S))
            {
                if (fakeSpeed < -this.maxFakeSpeed)
                {
                    fakeSpeed = -maxFakeSpeed;
                }
                else
                {
                    fakeSpeed += -fakeSpeedDecrement;
                }
                rigidbody.velocity = fakeSpeed * transform.forward;
            }
            playerConverter.ToLatLon(transform.position.x + x, transform.position.z + z, (int)playerConverter.Zone, playerConverter.Hemi);
            fakeLat = playerConverter.Latitude;
            fakeLon = playerConverter.Longitude;
        }




    }
}
