﻿/*
 * The MIT License (MIT)
Copyright (c) 2013 João Tiago Pinheiro Neto Jacob
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*
*/
using UnityEngine;
using System.Collections;

public class UserScript : Player {

    public float cameraFOV = 28.0F;
    public float compassOffset = 0.0F;
    public float distanceToTarget;
    public float userHeight;
    public Transform prefabTarget;
    private float initialX;
    private float initialZ;
    public Vector2 targetCoordinates;
    public float targetHeight;
	// Use this for initialization
	new IEnumerator Start () {

        Screen.sleepTimeout = SleepTimeout.NeverSleep;
        
        if (Input.location.isEnabledByUser)
        {
            isReady = false;
            Debug.Log("Location Available");
            Input.location.Start();
            int maxWait = 20;
            while (Input.location.status == LocationServiceStatus.Initializing && maxWait > 0)
            {
                Debug.Log("Waiting For Location");
                yield return new WaitForSeconds(1);
                maxWait--;
            }
            if (maxWait < 1)
                print("Timed out");
            if (Input.location.status == LocationServiceStatus.Failed)
            {
               print("Unable to determine device location");
            }
            else if (Input.location.status == LocationServiceStatus.Running)
            {
                isReady = true;
                Debug.Log("Current user Location was caught");
                print("Location: " + Input.location.lastData.latitude + " " + Input.location.lastData.longitude + " " + Input.location.lastData.altitude + " " + Input.location.lastData.horizontalAccuracy + " " + Input.location.lastData.timestamp);
                initialFakeLat = Input.location.lastData.latitude;
                initialFakeLon = Input.location.lastData.longitude;
                GeoUTMConverter playerConverter = new GeoUTMConverter();
                playerConverter.ToUTM((double)initialFakeLat, (double)initialFakeLon);
                initialX = (float)playerConverter.X;
                initialZ = (float)playerConverter.Y;
            }
        }

        if (Application.platform == RuntimePlatform.Android)
        {
            GetComponent<CustomGyroController>().enabled = true;
        }
        else
        {
           GetComponent<CustomGyroController>().enabled = false;
           GeoUTMConverter playerConverter = new GeoUTMConverter();
           playerConverter.ToUTM((double)initialFakeLat, (double)initialFakeLon);
           initialX = (float)playerConverter.X;
           initialZ = (float)playerConverter.Y;
           isReady = true;
            
        }
	}

    void OnGUI()
    {
        GUIStyle TextStyle = GUI.skin.label;
        TextStyle.fontSize = 30;
        TextStyle.richText = true;

        GUIStyle ThumbStyle = GUI.skin.horizontalSliderThumb;
        ThumbStyle.stretchHeight = true;
        ThumbStyle.stretchWidth = true;
        ThumbStyle.fixedHeight = 50;
        ThumbStyle.fixedWidth = 50;
        //User Stuff
        {
            GUI.Label(new Rect(Screen.width / 2 + 25, Screen.height - 150, 300, 100), "User", TextStyle);

            GUI.Label(new Rect(Screen.width / 2 - 300, Screen.height - 100, 300, 100), "Lat: " + fakeLat.ToString().Substring(0, Mathf.Min(9, fakeLat.ToString().Length)), TextStyle);

            GUI.Label(new Rect(Screen.width / 2, Screen.height - 100, 300, 100), "Lon: " + fakeLon.ToString().Substring(0, Mathf.Min(9, fakeLon.ToString().Length)), TextStyle);

            GUI.Label(new Rect(Screen.width / 2 + 300, Screen.height - 100, 300, 100), "Altitude: " + userHeight.ToString().Substring(0, Mathf.Min(9, userHeight.ToString().Length)), TextStyle);

            GUI.Label(new Rect(Screen.width / 2 , Screen.height - 50, 300, 100), "Yaw: " + transform.eulerAngles.y, TextStyle);
        }

        //Target Stuff
        {
            GUI.Label(new Rect(Screen.width / 2 + 25, 150, 300, 100), "Target", TextStyle);

            GUI.Label(new Rect(Screen.width / 2 - 300, 100, 300, 100), "Lat: " + targetCoordinates.x.ToString().Substring(0, Mathf.Min(9, targetCoordinates.x.ToString().Length)), TextStyle);

            GUI.Label(new Rect(Screen.width / 2, 100, 300, 100), "Lon: " + targetCoordinates.y.ToString().Substring(0, Mathf.Min(9, targetCoordinates.y.ToString().Length)), TextStyle);

            GUI.Label(new Rect(Screen.width / 2 + 300, 100, 300, 100), "Altitude: " + targetHeight.ToString().Substring(0, Mathf.Min(9, targetHeight.ToString().Length)), TextStyle);
        }

        //Calibration Stuff
        {
            cameraFOV = GUI.HorizontalSlider(new Rect(Screen.width / 2 - 200, 50, 200, 200), cameraFOV, 20, 60,GUI.skin.horizontalSlider,ThumbStyle);

            GUI.Label(new Rect(Screen.width / 2 - 200, 200, 200, 100), "FOV: " + cameraFOV, TextStyle);

            compassOffset = GUI.HorizontalSlider(new Rect(Screen.width / 2 + 200, 50, 200, 200), compassOffset, -10, 10, GUI.skin.horizontalSlider, ThumbStyle);

            GUI.Label(new Rect(Screen.width / 2 + 200, 200, 200, 100), "Compass Offset: " + compassOffset, TextStyle);

        }

        //Button Stuff
        {


            var value = "";
            if (GetComponent<CustomGyroController>().enabled)
            {
                value = "Gyro: On";
            }
            else
            {
                value = "Gyro: Off";
            }

            if (GUI.Button(new Rect(Screen.width - 150, Screen.height - 150, 100, 100), value))
            {
                if (GetComponent<CustomGyroController>().enabled)
                    GetComponent<CustomGyroController>().enabled = false;
                else
                    GetComponent<CustomGyroController>().enabled = true;
            }

        }


        //GUI.Label(new Rect(Screen.width/2 - 50, Screen.height/2 - 50, 100, 100), distanceToTarget.ToString());
    }


	
	// Update is called once per frame
	void Update () {

        this.camera.fieldOfView = cameraFOV;

        GetComponent<CustomGyroController>().compassOffset = compassOffset;

        GeoUTMConverter playerConverter = new GeoUTMConverter();

        if (Input.location.status == LocationServiceStatus.Running)
        {
            fakeLat = Input.location.lastData.latitude;
            fakeLon = Input.location.lastData.longitude;
            
            playerConverter.ToUTM((double)fakeLat, (double)fakeLon);
            x = (float)(playerConverter.X - initialX);
            z = (float)(playerConverter.Y - initialZ);
            transform.position = new Vector3(x, 0, z);
        }
        Vector3 position = new Vector3(transform.position.x, 10000, transform.position.z);
        
        RaycastHit hitInfo;
       
        Debug.DrawRay(transform.position, transform.forward * 10000.0f);
        if (Physics.Raycast(position, -Vector3.up, out hitInfo))
        {
            position.y = hitInfo.point.y + 1.5f;
            transform.position = position;
            userHeight = position.y;
        }
        if (Physics.Raycast(transform.position, transform.forward, out hitInfo))
        {
            distanceToTarget = Vector3.Distance(transform.position, hitInfo.point);
            prefabTarget.position = hitInfo.point;
            targetHeight = hitInfo.point.y;
            

            playerConverter.ToLatLon((double)(initialX + hitInfo.point.x), (double)(initialZ + hitInfo.point.z), (int)playerConverter.Zone, GeoUTMConverter.Hemisphere.Northern);
            targetCoordinates.x = (float)playerConverter.Latitude;
            targetCoordinates.y = (float)playerConverter.Longitude;

        }
        else
        {
            targetHeight = -1;
            distanceToTarget = -1;
            targetCoordinates.x = -1;
            targetCoordinates.y = -1;
        }

        
      
        
	}
}
